@@ -31,7 +31,7 @@ extern "C" {
 
 #include <connman/property.h>
 #include <connman/types.h>
-#include <connman/ipv4.h>
+#include <connman/ipconfig.h>
 
 /**
  * SECTION:element
@@ -56,6 +56,20 @@ enum connman_element_type {
 	CONNMAN_ELEMENT_TYPE_VENDOR     = 10000,
 };
 
+enum connman_element_state {
+	CONNMAN_ELEMENT_STATE_UNKNOWN = 0,
+	CONNMAN_ELEMENT_STATE_ERROR   = 1,
+	CONNMAN_ELEMENT_STATE_IDLE    = 2,
+	CONNMAN_ELEMENT_STATE_DONE    = 3,
+};
+
+enum connman_element_error {
+	CONNMAN_ELEMENT_ERROR_UNKNOWN        = 0,
+	CONNMAN_ELEMENT_ERROR_FAILED         = 1,
+	CONNMAN_ELEMENT_ERROR_DHCP_FAILED    = 2,
+	CONNMAN_ELEMENT_ERROR_CONNECT_FAILED = 3,
+};
+
 struct connman_driver;
 
 struct connman_element {
@@ -64,6 +78,8 @@ struct connman_element {
 	gchar *name;
 	gchar *path;
 	enum connman_element_type type;
+	enum connman_element_state state;
+	enum connman_element_error error;
 	gboolean enabled;
 	gboolean configuring;
 	gchar *devname;
@@ -84,7 +100,7 @@ struct connman_element {
 	GHashTable *properties;
 
 	struct {
-		enum connman_ipv4_method method;
+		enum connman_ipconfig_method method;
 		gchar *address;
 		gchar *netmask;
 		gchar *gateway;
@@ -94,34 +110,36 @@ struct connman_element {
 	} ipv4;
 };
 
-extern struct connman_element *connman_element_create(const char *name);
-extern struct connman_element *connman_element_ref(struct connman_element *element);
-extern void connman_element_unref(struct connman_element *element);
+struct connman_element *connman_element_create(const char *name);
+struct connman_element *connman_element_ref(struct connman_element *element);
+void connman_element_unref(struct connman_element *element);
 
-extern int connman_element_get_value(struct connman_element *element,
+int connman_element_get_value(struct connman_element *element,
 				enum connman_property_id id, void *value);
 
-extern int connman_element_set_string(struct connman_element *element,
+int connman_element_set_string(struct connman_element *element,
 					const char *key, const char *value);
-extern const char *connman_element_get_string(struct connman_element *element,
+const char *connman_element_get_string(struct connman_element *element,
 							const char *key);
-extern int connman_element_set_uint8(struct connman_element *element,
+int connman_element_set_uint8(struct connman_element *element,
 				const char *key, connman_uint8_t value);
-extern connman_uint8_t connman_element_get_uint8(struct connman_element *element,
+connman_uint8_t connman_element_get_uint8(struct connman_element *element,
 							const char *key);
-extern int connman_element_set_blob(struct connman_element *element,
+int connman_element_set_blob(struct connman_element *element,
 			const char *key, const void *data, unsigned int size);
-extern const void *connman_element_get_blob(struct connman_element *element,
+const void *connman_element_get_blob(struct connman_element *element,
 					const char *key, unsigned int *size);
 
-extern int connman_element_register(struct connman_element *element,
+int connman_element_register(struct connman_element *element,
 					struct connman_element *parent);
-extern void connman_element_unregister(struct connman_element *element);
-extern void connman_element_unregister_children(struct connman_element *element);
-extern void connman_element_update(struct connman_element *element);
+void connman_element_unregister(struct connman_element *element);
+void connman_element_unregister_children(struct connman_element *element);
+void connman_element_update(struct connman_element *element);
 
-extern int connman_element_set_enabled(struct connman_element *element,
+int connman_element_set_enabled(struct connman_element *element,
 							gboolean enabled);
+void connman_element_set_error(struct connman_element *element,
+					enum connman_element_error error);
 
 static inline void *connman_element_get_data(struct connman_element *element)
 {
