@@ -27,44 +27,8 @@
 
 #include "connman.h"
 
-enum connman_policy {
-	CONNMAN_POLICY_UNKNOWN  = 0,
-	CONNMAN_POLICY_SINGLE   = 1,
-	CONNMAN_POLICY_MULTIPLE = 2,
-	CONNMAN_POLICY_ASK      = 3,
-};
-
-static enum connman_policy global_policy = CONNMAN_POLICY_SINGLE;
 static connman_bool_t global_offlinemode = FALSE;
 
-static const char *policy2string(enum connman_policy policy)
-{
-	switch (policy) {
-	case CONNMAN_POLICY_UNKNOWN:
-		break;
-	case CONNMAN_POLICY_SINGLE:
-		return "single";
-	case CONNMAN_POLICY_MULTIPLE:
-		return "multiple";
-	case CONNMAN_POLICY_ASK:
-		return "ask";
-	}
-
-	return NULL;
-}
-
-static enum connman_policy string2policy(const char *policy)
-{
-	if (g_str_equal(policy, "single") == TRUE)
-		return CONNMAN_POLICY_SINGLE;
-	else if (g_str_equal(policy, "multiple") == TRUE)
-		return CONNMAN_POLICY_MULTIPLE;
-	else if (g_str_equal(policy, "ask") == TRUE)
-		return CONNMAN_POLICY_ASK;
-	else
-		return CONNMAN_POLICY_UNKNOWN;
-}
-
 static void append_profiles(DBusMessageIter *dict)
 {
 	DBusMessageIter entry, value, iter;
@@ -161,6 +125,35 @@ static void append_connections(DBusMessageIter *dict)
 	dbus_message_iter_close_container(dict, &entry);
 }
 
+static void append_technologies(gboolean powered, DBusMessageIter *dict)
+{
+	DBusMessageIter entry, value, iter;
+	const char *key;
+
+	if (powered)
+		key = "EnabledTechnologies";
+	else
+		key = "Technologies";
+
+	dbus_message_iter_open_container(dict, DBUS_TYPE_DICT_ENTRY,
+								NULL, &entry);
+
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+			DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_STRING_AS_STRING,
+								&value);
+
+	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
+					DBUS_TYPE_STRING_AS_STRING, &iter);
+	__connman_notifier_device_type_list(powered, &iter);
+	dbus_message_iter_close_container(&value, &iter);
+
+	dbus_message_iter_close_container(&entry, &value);
+
+	dbus_message_iter_close_container(dict, &entry);
+}
+
 static DBusMessage *get_properties(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
@@ -204,14 +197,12 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	connman_dbus_dict_append_variant(&dict, "State",
 						DBUS_TYPE_STRING, &str);
 
-	str = policy2string(global_policy);
-	if (str != NULL)
-		connman_dbus_dict_append_variant(&dict, "Policy",
-						DBUS_TYPE_STRING, &str);
-
 	connman_dbus_dict_append_variant(&dict, "OfflineMode",
 				DBUS_TYPE_BOOLEAN, &global_offlinemode);
 
+	append_technologies(FALSE, &dict);
+	append_technologies(TRUE, &dict);
+
 	dbus_message_iter_close_container(&array, &dict);
 
 	return reply;
@@ -236,23 +227,13 @@ static DBusMessage *set_property(DBusConnection *conn,
 					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
 		return __connman_error_permission_denied(msg);
 
-	if (g_str_equal(name, "Policy") == TRUE) {
-		enum connman_policy policy;
-		const char *str;
-
-		dbus_message_iter_get_basic(&value, &str);
-		policy = string2policy(str);
-		if (policy == CONNMAN_POLICY_UNKNOWN)
-			return __connman_error_invalid_arguments(msg);
-
-		global_policy = policy;
-	} else if (g_str_equal(name, "OfflineMode") == TRUE) {
+	if (g_str_equal(name, "OfflineMode") == TRUE) {
 		connman_bool_t offlinemode;
 
 		dbus_message_iter_get_basic(&value, &offlinemode);
 
 		if (global_offlinemode == offlinemode)
-			return __connman_error_invalid_arguments(msg);
+			return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 
 		global_offlinemode = offlinemode;
 
@@ -263,7 +244,8 @@ static DBusMessage *set_property(DBusConnection *conn,
 		dbus_message_iter_get_basic(&value, &str);
 
 		return __connman_error_not_supported(msg);
-	}
+	} else
+		return __connman_error_invalid_property(msg);
 
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
@@ -294,10 +276,70 @@ static DBusMessage *remove_profile(DBusConnection *conn,
 	return __connman_error_not_supported(msg);
 }
 
+static DBusMessage *request_scan(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	enum connman_device_type type;
+	const char *str;
+	int err;
+
+	DBG("conn %p", conn);
+
+	dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &str,
+							DBUS_TYPE_INVALID);
+
+	if (g_strcmp0(str, "") == 0)
+		type = CONNMAN_DEVICE_TYPE_UNKNOWN;
+	else if (g_strcmp0(str, "wifi") == 0)
+		type = CONNMAN_DEVICE_TYPE_WIFI;
+	else if (g_strcmp0(str, "wimax") == 0)
+		type = CONNMAN_DEVICE_TYPE_WIMAX;
+	else
+		return __connman_error_invalid_arguments(msg);
+
+	err = __connman_element_request_scan(type);
+	if (err < 0) {
+		if (err == -EINPROGRESS) {
+			connman_error("Invalid return code from scan");
+			err = -EINVAL;
+		}
+
+		return __connman_error_failed(msg, -err);
+	}
+
+	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
+}
+
+static DBusMessage *enable_technology(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	const char *str;
+
+	DBG("conn %p", conn);
+
+	dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &str,
+							DBUS_TYPE_INVALID);
+
+	return __connman_error_not_supported(msg);
+}
+
+static DBusMessage *disable_technology(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	const char *str;
+
+	DBG("conn %p", conn);
+
+	dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &str,
+							DBUS_TYPE_INVALID);
+
+	return __connman_error_not_supported(msg);
+}
+
 static DBusMessage *connect_service(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
-	DBusMessageIter iter, array;
+	int err;
 
 	DBG("conn %p", conn);
 
@@ -305,26 +347,17 @@ static DBusMessage *connect_service(DBusConnection *conn,
 					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
 		return __connman_error_permission_denied(msg);
 
-	dbus_message_iter_init(msg, &iter);
-	dbus_message_iter_recurse(&iter, &array);
-
-	while (dbus_message_iter_get_arg_type(&array) == DBUS_TYPE_DICT_ENTRY) {
-		DBusMessageIter entry, value;
-		const char *key;
-
-		dbus_message_iter_recurse(&array, &entry);
-		dbus_message_iter_get_basic(&entry, &key);
-
-		dbus_message_iter_next(&entry);
-		dbus_message_iter_recurse(&entry, &value);
-
-		switch (dbus_message_iter_get_arg_type(&value)) {
+	err = __connman_service_create_and_connect(msg);
+	if (err < 0) {
+		if (err == -EINPROGRESS) {
+			connman_error("Invalid return code from connect");
+			err = -EINVAL;
 		}
 
-		dbus_message_iter_next(&array);
+		return __connman_error_failed(msg, -err);
 	}
 
-	return __connman_error_not_implemented(msg);
+	return NULL;
 }
 
 static DBusMessage *register_agent(DBusConnection *conn,
@@ -376,18 +409,23 @@ static DBusMessage *unregister_agent(DBusConnection *conn,
 }
 
 static GDBusMethodTable manager_methods[] = {
-	{ "GetProperties",   "",      "a{sv}", get_properties   },
-	{ "SetProperty",     "sv",    "",      set_property     },
-	{ "AddProfile",      "s",     "o",     add_profile      },
-	{ "RemoveProfile",   "o",     "",      remove_profile   },
-	{ "ConnectService",  "a{sv}", "o",     connect_service  },
-	{ "RegisterAgent",   "o",     "",      register_agent   },
-	{ "UnregisterAgent", "o",     "",      unregister_agent },
+	{ "GetProperties",     "",      "a{sv}", get_properties     },
+	{ "SetProperty",       "sv",    "",      set_property       },
+	{ "AddProfile",        "s",     "o",     add_profile        },
+	{ "RemoveProfile",     "o",     "",      remove_profile     },
+	{ "RequestScan",       "s",     "",      request_scan       },
+	{ "EnableTechnology",  "s",     "",      enable_technology  },
+	{ "DisableTechnology", "s",     "",      disable_technology },
+	{ "ConnectService",    "a{sv}", "o",     connect_service,
+						G_DBUS_METHOD_FLAG_ASYNC },
+	{ "RegisterAgent",     "o",     "",      register_agent     },
+	{ "UnregisterAgent",   "o",     "",      unregister_agent   },
 	{ },
 };
 
 static GDBusSignalTable manager_signals[] = {
 	{ "PropertyChanged", "sv" },
+	{ "StateChanged",    "s"  },
 	{ },
 };
 
